==> asgi_tools-0.0.22-py3-none-any.whl: asgi_tools/middleware.py <==
"""ASGI-Tools Middlewares."""

from functools import partial

from http_router import Router

from .request import Request
from .response import HTMLResponse, parse_response
from .utils import to_coroutine


class BaseMiddeware:
    """Base class for ASGI-Tools middlewares."""

    scopes = {'http', 'websockets'}

    def __init__(self, app=None, **kwargs):
        """Save ASGI App."""

        self.bind(app)

    async def __call__(self, scope, *args, **kwargs):
        """Handle ASGI call."""

        if scope['type'] in self.scopes:
            return await self.__process__(scope, *args, **kwargs)

        return await self.app(scope, *args, **kwargs)

    def __getattr__(self, name):
        """Proxy middleware methods to root level."""

        return getattr(self.app, name)

    async def __process__(self, scope, receive, send):
        """Do the middleware's logic."""

        raise NotImplementedError()

    def bind(self, app=None):
        """Bind the middleware to an ASGI application."""
        self.app = app or HTMLResponse("Not Found", status_code=404)
        return self


class ResponseMiddleware(BaseMiddeware):
    """Parse different responses from ASGI applications."""

    async def __process__(self, scope, receive, send):
        """Parse responses from callbacks."""

        response = await self.app(scope, receive, send)
        if response is not None:
            response = await parse_response(response)

            # Send ASGI messages from the prepared response
            async for msg in response:
                await send(msg)


class RequestMiddleware(BaseMiddeware):
    """Provider asgi_tools.Request to apps."""

    async def __process__(self, scope, receive, send):
        """Replace scope with request object."""
        return await self.app(Request(scope, receive, send), receive, send)


class LifespanMiddleware(BaseMiddeware):
    """Manage lifespan events."""

    scopes = {'lifespan'}

    def __init__(self, app=None, on_startup=None, on_shutdown=None, **kwargs):
        """Prepare the middleware."""
        super(LifespanMiddleware, self).__init__(app, **kwargs)
        self._startup = []
        self._shutdown = []
        self.__register__(on_startup, self._startup)
        self.__register__(on_shutdown, self._shutdown)

    async def __process__(self, scope, receive, send):
        """Manage lifespan cycle."""
        while True:
            message = await receive()
            if message['type'] == 'lifespan.startup':
                await self.__startup__(scope)
                await send({'type': 'lifespan.startup.complete'})

            elif message['type'] == 'lifespan.shutdown':
                await self.__shutdown__(scope)
                return await send({'type': 'lifespan.shutdown.complete'})

    def __register__(self, handlers, container):
        """Register lifespan handlers."""
        if not handlers:
            return

        if not isinstance(handlers, list):
            handlers = [handlers]

        container += [to_coroutine(fn) for fn in handlers]
        return container

    async def __startup__(self, scope):
        """Run startup callbacks."""
        for fn in self._startup:
            await fn()

    async def __shutdown__(self, scope):
        """Run shutdown callbacks."""
        for fn in self._shutdown:
            await fn()

    def on_startup(self, fn):
        """Add a function to startup."""
        self.__register__(fn, self._startup)

    def on_shutdown(self, fn):
        """Add a function to shutdown."""
        self.__register__(fn, self._shutdown)


class RouterMiddleware(BaseMiddeware):
    """Bind callbacks to HTTP paths."""

    def __init__(self, app=None, router: Router = None, **kwargs):
        """Initialize HTTP router. """
        super(RouterMiddleware, self).__init__(app, **kwargs)
        self.router = router

    async def __process__(self, scope, receive, send):
        """Get an app and process."""
        app, matches = self.__dispatch__(scope)
        scope['matches'] = matches
        return await app(scope, receive, send)

    def __dispatch__(self, scope):
        """Lookup for a callback."""
        try:
            return self.router(scope.get("root_path", "") + scope["path"], scope['method'])
        except self.router.NotFound:
            return self.app, {}


def AppMiddleware(
        app=None, *app_middlewares, pass_matches_only=True, parse_response=True, **params):
    """Combine middlewares to create an application."""

    async def default404(request, **params):
        return HTMLResponse("Not Found", status_code=404)

    middlewares = [
        LifespanMiddleware, RequestMiddleware, ResponseMiddleware, *app_middlewares,
        partial(
            RouterMiddleware, pass_matches_only=pass_matches_only, parse_response=parse_response)
    ]
    return combine(app or default404, *middlewares, **params)


def combine(app, *middlewares, **params):
    """Combine the given middlewares into the given application."""

    for md in list(middlewares)[::-1]:
        app = md(app, **params)

    return app

==> asgi_tools-0.1.4-py3-none-any.whl: asgi_tools/middleware.py <==
"""ASGI-Tools Middlewares."""

from functools import partial

from http_router import Router

from .request import Request
from .response import ResponseHTML, parse_response, ResponseError
from .utils import to_coroutine


class BaseMiddeware:
    """Base class for ASGI-Tools middlewares."""

    scopes = {'http', 'websockets'}

    def __init__(self, app=None, **kwargs):
        """Save ASGI App."""

        self.bind(app)

    async def __call__(self, scope, *args, **kwargs):
        """Handle ASGI call."""

        if scope['type'] in self.scopes:
            return await self.__process__(scope, *args, **kwargs)

        return await self.app(scope, *args, **kwargs)

    def __getattr__(self, name):
        """Proxy middleware methods to root level."""

        return getattr(self.app, name)

    async def __process__(self, scope, receive, send):
        """Do the middleware's logic."""

        raise NotImplementedError()

    def bind(self, app=None):
        """Bind the middleware to an ASGI application."""
        self.app = app or ResponseHTML("Not Found", status_code=404)
        return self


class ResponseMiddleware(BaseMiddeware):
    """Parse different responses from ASGI applications."""

    async def __process__(self, scope, receive, send):
        """Parse responses from callbacks."""

        try:
            response = await self.app(scope, receive, send)
            if response is None:
                return

            response = await parse_response(response)

        except ResponseError as exc:
            response = exc

        # Send ASGI messages from the prepared response
        async for msg in response:
            await send(msg)


class RequestMiddleware(BaseMiddeware):
    """Provider asgi_tools.Request to apps."""

    async def __process__(self, scope, receive, send):
        """Replace scope with request object."""
        return await self.app(Request(scope, receive, send), receive, send)


class LifespanMiddleware(BaseMiddeware):
    """Manage lifespan events."""

    scopes = {'lifespan'}

    def __init__(self, app=None, on_startup=None, on_shutdown=None, **kwargs):
        """Prepare the middleware."""
        super(LifespanMiddleware, self).__init__(app, **kwargs)
        self._startup = []
        self._shutdown = []
        self.__register__(on_startup, self._startup)
        self.__register__(on_shutdown, self._shutdown)

    async def __process__(self, scope, receive, send):
        """Manage lifespan cycle."""
        while True:
            message = await receive()
            if message['type'] == 'lifespan.startup':
                await self.__startup__(scope)
                await send({'type': 'lifespan.startup.complete'})

            elif message['type'] == 'lifespan.shutdown':
                await self.__shutdown__(scope)
                return await send({'type': 'lifespan.shutdown.complete'})

    def __register__(self, handlers, container):
        """Register lifespan handlers."""
        if not handlers:
            return

        if not isinstance(handlers, list):
            handlers = [handlers]

        container += [to_coroutine(fn) for fn in handlers]
        return container

    async def __startup__(self, scope):
        """Run startup callbacks."""
        for fn in self._startup:
            await fn()

    async def __shutdown__(self, scope):
        """Run shutdown callbacks."""
        for fn in self._shutdown:
            await fn()

    def on_startup(self, fn):
        """Add a function to startup."""
        self.__register__(fn, self._startup)

    def on_shutdown(self, fn):
        """Add a function to shutdown."""
        self.__register__(fn, self._shutdown)


class RouterMiddleware(BaseMiddeware):
    """Bind callbacks to HTTP paths."""

    def __init__(self, app=None, router: Router = None, **kwargs):
        """Initialize HTTP router. """
        super(RouterMiddleware, self).__init__(app, **kwargs)
        self.router = router

    async def __process__(self, scope, receive, send):
        """Get an app and process."""
        app, matches = self.__dispatch__(scope)
        scope['matches'] = matches
        return await app(scope, receive, send)

    def __dispatch__(self, scope):
        """Lookup for a callback."""
        try:
            return self.router(scope.get("root_path", "") + scope["path"], scope['method'])
        except self.router.NotFound:
            return self.app, {}


def AppMiddleware(
        app=None, *app_middlewares, pass_matches_only=True, parse_response=True, **params):
    """Combine middlewares to create an application."""

    async def default404(request, **params):
        return ResponseHTML("Not Found", status_code=404)

    middlewares = [
        LifespanMiddleware, RequestMiddleware, ResponseMiddleware, *app_middlewares,
        partial(
            RouterMiddleware, pass_matches_only=pass_matches_only, parse_response=parse_response)
    ]
    return combine(app or default404, *middlewares, **params)


def combine(app, *middlewares, **params):
    """Combine the given middlewares into the given application."""

    for md in list(middlewares)[::-1]:
        app = md(app, **params)

    return app
